import { Edge, Graph, Vertex } from "../graph.interface";
import { Citation } from "./citation";
import { LedgerEntry, Member } from "./member";
import { Post } from "./post";

export class Forum implements Graph {

    private posts = new Map<string, Post>();
    private citations = new Map<string, Citation>();
    private members = new Map<string, Member>();
    private ledger = new Map<string, LedgerEntry>();

    //TODO: Need to define the forum parameters
    public params = {};

    constructor(params: object, posts: Post[], citations: Citation[], members: Member[]) {
        this.params = params;
        posts.forEach(post => this.posts.set(post.id, post));
        citations.forEach(citation => this.citations.set(citation.id, citation));
        members.forEach(member => this.members.set(member.id, member));
    }

    public get vertices(): Map<string, Vertex> { return this.posts; }
    public get edges(): Map<string, Edge> { return this.citations; }

    public getAllVertices(): Vertex[] { return this.getAllPosts(); }
    public getVertex(id: string): Vertex { return this.getPost(id); }
    public addVertex(vertex: Vertex): void { return this.addPost(vertex as Post); }
    public deleteVertex(vertex: string | Vertex): void { return this.deletePost(vertex as any); }

    public getAllEdges(): Edge[] { return this.getAllCitations(); }
    public getEdge(id: string): Edge { return this.getCitation(id); }
    public addEdge(edge: Edge): void { return this.addCitation(edge as Citation); }
    public deleteEdge(edge: string | Edge): void { return this.deleteCitation(edge as any); }

    public getAllPosts() {
        return Array.from(this.posts.values());
    }

    public getPost(id: string) {
        return this.posts.get(id);
    }

    public addPost(post: Post) {
        this.posts.set(post.id, post);
    }

    public deletePost(post: string | Post) {
        if (typeof post === "string") post = this.posts.get(post);
        for (const citation of post.citations) this.deleteCitation(citation);
        this.posts.delete(post.id);
    }

    public getAllCitations() {
        return Array.from(this.citations.values());
    }

    public getCitation(id: string) {
        return this.citations.get(id);
    }

    public addCitation(citation: Citation) {
        citation.sourcePost.setCitation(citation);
        citation.citedPost.setCitation(citation);
        this.citations.set(citation.id, citation);
    }

    public deleteCitation(citation: string | Citation) {
        if (typeof citation === "string") citation = this.citations.get(citation);
        this.citations.delete(citation.id);
        citation.sourcePost.removeCitation(citation);
        citation.citedPost.removeCitation(citation);
    }

    public getAllMembers() {
        return Array.from(this.members.values());
    }

    public getMember(id: string) {
        return this.members.get(id);
    }

    public addMember(member: Member) {
        this.members.set(member.id, member);
    }

    public deleteMember(member: string | Member) {
        if (typeof member === "string") member = this.members.get(member);
        this.members.delete(member.id);
    }

    //TODO: Need to define this function
    public getBranch(post: Post, depth: number, direction: 'predecessor' | 'successor') {

    }

    //TODO: Need to define this function
    public assesPostImpact(post: Post, depth: number = 1) {

    }

    //TODO: Need to define this function
    public propagateReputation(post: Post) {

    }

} 